/**
 * 设备信息
 * @type {string}
 */
(function (window, document) {

  window.DEVICE = {
    ios: false,
    android: false,
    androidChrome: false,
    desktop: false,
    windowsPhone: false,
    iphone: false,
    iphoneX: false,
    ipod: false,
    ipad: false,
    edge: false,
    ie: false,
    isWeiXin: false,
    isWeiXinAndroid: false,
    isWeiXiniOS: false,
    isQQ: false,
    isQQAndroid: false,
    isQQiOS: false,
    isWeiBo: false,
    isWeiBoAndroid: false,
    isWeiBoiOS: false,
    firefox: false,
    macos: false,
    windows: false,
    app: 'browser',
    mode: 'browser',
    cordova: !!(window.cordova || window.phonegap),
    phonegap: !!(window.cordova || window.phonegap),
  };
  var platform = window.navigator.platform;
  var ua = window.navigator.userAgent;
  var screenWidth = window.screen.width;
  var screenHeight = window.screen.height;
  var windowsPhone = ua.match(/(Windows Phone);?[\s\/]+([\d.]+)?/); // eslint-disable-line
  var android = ua.match(/(Android);?[\s\/]+([\d.]+)?/) || ua.match(/(Android);?([\d.]+)?/); // eslint-disable-line
  var ipad = ua.match(/(iPad).*OS\s([\d_]+)/);
  var ipod = ua.match(/(iPod)(.*OS\s([\d_]+))?/);
  var iphone = !ipad && ua.match(/(iPhone\sOS|iOS)\s([\d_]+)/);
  var iphoneX = iphone && (
    (screenWidth === 375 && screenHeight === 812) // X/XS
      || (screenWidth === 414 && screenHeight === 896) // XR / XS Max
  );
  var ie = ua.indexOf('MSIE ') >= 0 || ua.indexOf('Trident/') >= 0;
  var edge = ua.indexOf('Edge/') >= 0;
  var firefox = ua.indexOf('Gecko/') >= 0 && ua.indexOf('Firefox/') >= 0;
  var macos = platform === 'MacIntel';
  var windows = platform === 'Win32';
  DEVICE.ua = ua;
  DEVICE.ie = ie;
  DEVICE.edge = edge;
  DEVICE.firefox = firefox;
  DEVICE.webView = !!((iphone || ipad || ipod) && (ua.match(/.*AppleWebKit(?!.*Safari)/i) || window.navigator.standalone))
      || (window.matchMedia && window.matchMedia('(display-mode: standalone)').matches);
  // Windows
  if (windowsPhone) {
    DEVICE.os = 'windows';
    DEVICE.osVersion = windows[2];
    DEVICE.windowsPhone = true;
  }
  // Android
  if (android && !windows) {
    DEVICE.os = 'android';
    DEVICE.osVersion = android[2];
    DEVICE.android = true;
    DEVICE.androidChrome = ua.toLowerCase().indexOf('chrome') >= 0;
  }
  if (ipad || iphone || ipod) {
    DEVICE.os = 'ios';
    DEVICE.ios = true;
  }
  // iOS
  if (iphone && !ipod) {
    DEVICE.osVersion = iphone[2].replace(/_/g, '.');
    DEVICE.iphone = true;
    DEVICE.iphoneX = iphoneX;
  }
  if (ipad) {
    DEVICE.osVersion = ipad[2].replace(/_/g, '.');
    DEVICE.ipad = true;
  }
  if (ipod) {
    DEVICE.osVersion = ipod[3] ? ipod[3].replace(/_/g, '.') : null;
    DEVICE.iphone = true;
  }
  // iOS 8+ changed UA
  if (DEVICE.ios && DEVICE.osVersion && ua.indexOf('Version/') >= 0) {
    if (DEVICE.osVersion.split('.')[0] === '10') {
      DEVICE.osVersion = ua.toLowerCase().split('version/')[1].split(' ')[0];
    }
  }
  // Desktop
  DEVICE.desktop = !(DEVICE.os || DEVICE.android);
  if (DEVICE.desktop) {
    DEVICE.macos = macos;
    DEVICE.windows = windows;
  }
  DEVICE.isWeiXin = /MicroMessenger/i.test(ua);
  DEVICE.isPiPiApp = /pipipeiwan/i.test(ua);
  DEVICE.isPiPiAppiOS = DEVICE.isPiPiApp && DEVICE.ios;
  DEVICE.isPiPiAppAndriod = DEVICE.isPiPiApp && DEVICE.android;
  DEVICE.isWeiXinAndroid = DEVICE.isWeiXin && DEVICE.android;
  DEVICE.isWeiXiniOS = DEVICE.isWeiXin && DEVICE.ios;
  DEVICE.isWeiBo = /WeiBo/i.test(ua);
  DEVICE.isWeiBoAndroid = DEVICE.isWeiBo && DEVICE.android;
  DEVICE.isWeiBoiOS = DEVICE.isWeiBo && DEVICE.ios;
  DEVICE.isQQ = / QQ/i.test(ua);
  DEVICE.isQQiOS = DEVICE.isQQ && DEVICE.ios;
  DEVICE.isQQAndroid = DEVICE.isQQ && DEVICE.android;
  DEVICE.isJym = ua.indexOf('jym_mobile') !== -1 || ua.indexOf('JymApp') !== -1; // 交易猫的标识
  DEVICE.isJymiOS = DEVICE.isJym && DEVICE.ios;
  DEVICE.isJymAndroid = DEVICE.isJym && DEVICE.android;
  var classNames = [];
  var html = document.querySelector('html');
  // Pixel Ratio
  DEVICE.pixelRatio = window.devicePixelRatio || 1;
  // Pixel Ratio
  classNames.push('device-pixel-ratio-' + Math.floor(DEVICE.pixelRatio));
  if (DEVICE.pixelRatio >= 2) {
    classNames.push('device-retina');
  }
  if (DEVICE.isWeiXin) {
    DEVICE.app = 'weixin';
    classNames.push('app-weixin');
  }
  if (DEVICE.isQQ) {
    DEVICE.app = 'qq';
    classNames.push('app-qq');
  }
  if (DEVICE.isWeiBo) {
    DEVICE.app = 'weibo';
    classNames.push('app-weibo');
  }
  if (DEVICE.isPiPiApp) {
    DEVICE.app = 'pipi';
    classNames.push('app-pipi');
  }
  if (DEVICE.isJym) {
    DEVICE.app = 'jym';
    DEVICE.mode = 'jym';
    classNames.push('app-jym');
    if (DEVICE.iphone) {
      classNames.push(DEVICE.iphoneX ? 'app-jym-iphoneX' : 'app-jym-iphone');
    }
  }
  // OS classes
  if (DEVICE.os) {
    classNames.push(
      'device-' + DEVICE.os,
      'device-' + DEVICE.os + '-' + DEVICE.osVersion.split('.')[0],
      'device-' + DEVICE.os + '-' + DEVICE.osVersion.replace(/\./g, '-')
    );
    if (DEVICE.os === 'ios') {
      var major = parseInt(DEVICE.osVersion.split('.')[0], 10);
      for (var i = major - 1; i >= 6; i -= 1) {
        classNames.push('device-ios-gt-' + i);
      }
      if (DEVICE.iphoneX) {
        classNames.push('device-iphone-x');
      }
    }
  } else if (DEVICE.desktop) {
    classNames.push('device-desktop');
    if (DEVICE.macos) classNames.push('device-macos');
    else if (DEVICE.windows) classNames.push('device-windows');
  }
  // Add html classes
  for (var i = 0; i < classNames.length; i++) {
    html.classList.add(classNames[i]);
  }

  var minStyle = document.createElement('style');
  var minScreenHeight = window.screen.height + 'px';
  var minInnerHeight = window.innerHeight + 'px';
  minStyle.innerHTML = '.g-inner-full-height {height:' + minInnerHeight + '}  .g-screen-avail-height {height:' + window.screen.availHeight + 'px} .g-screen-full-height {height:' + minScreenHeight + '} .g-min-screen-full-height{min-height:' + minScreenHeight + '}  .g-min-inner-full-height{min-height:' + minInnerHeight + '}';
  document.getElementsByTagName('head').item(0).appendChild(minStyle);
})(window, document);

(function (window) {

  /**
   * todo token问题处理,因为token带有特殊字符 # ，容易导致单页应用路由问题，因此在存储和获取时需处理一下
   */
  window.getToken = function () {
    var getStorageToken = decodeURIComponent(localStorage.getItem('token') || sessionStorage.getItem('token'));
    return ['undefined', 'null'].indexOf(getStorageToken) === -1 ? getStorageToken : '';
  };

  /**
   * jquery extend
   * @type {function(): *}
   */
  window.extend = (function () {
    var isObjFunc = function (name) {
      var toString = Object.prototype.toString;
      return function () {
        return toString.call(arguments[0]) === '[object ' + name + ']';
      };
    };
    var isObject = isObjFunc('Object'),
      isArray = isObjFunc('Array'),
      isBoolean = isObjFunc('Boolean');
    return function extend() {
      var index = 0, isDeep = false, obj, copy, destination, source, i;
      if (isBoolean(arguments[0])) {
        index = 1;
        isDeep = arguments[0];
      }
      for (i = arguments.length - 1; i > index; i--) {
        destination = arguments[i - 1];
        source = arguments[i];
        if (isObject(source) || isArray(source)) {
          for (var property in source) {
            obj = source[property];
            if (isDeep && (isObject(obj) || isArray(obj))) {
              copy = isObject(obj) ? {} : [];
              var extended = extend(isDeep, copy, obj);
              destination[property] = extended;
            } else {
              destination[property] = source[property];
            }
          }
        } else {
          destination = source;
        }
      }
      return destination;
    };
  })();

  /**
   * 给 url 添加参数
   * @param {string} url
   * @param {string} stitchingUrlParams
   * @return {*}
   */
  window.addUrlParams = function (url, stitchingUrlParams) {
    var paramsPrefix = url.indexOf('?') !== -1 ? '&' : '?';
    return url + (stitchingUrlParams ? paramsPrefix : '') + stitchingUrlParams;
  };

  /**
   * 获取请求的url所有参数(包含search，和 hash)
   * @param url {string=} [window.location.href]
   */
  window.getRequestUrlParam = function (url) {
    var urlHashSearchStr = window.getUrlHashSearchStr(url);
    var hashStr = urlHashSearchStr.hashStr;
    var searchStr = urlHashSearchStr.searchStr;
    var hashStrParamsIndex = hashStr.indexOf('?');
    hashStr = hashStrParamsIndex !== -1 ? hashStr.substr(hashStrParamsIndex + 1) + '&' : '';
    var searchStrParamsIndex = searchStr.indexOf('?');
    if (searchStrParamsIndex !== -1) {
      searchStr = searchStr.substr(searchStrParamsIndex + 1);
    }
    var getRequestUrlParams = {};
    if (hashStrParamsIndex !== -1 || searchStrParamsIndex !== -1) {
      var paramsFormatUrl = (hashStr + searchStr).split('&');
      for (var i = 0; i < paramsFormatUrl.length; i++) {
        var param = paramsFormatUrl[i].split('=');
        if (param[0]) {
          var value = '';
          try {
            value = decodeURIComponent(param[1]);
          } catch (e) {
            value = param[1];
          }
          getRequestUrlParams[param[0]] = value;
        }
      }
    }
    return getRequestUrlParams;
  };

  /**
   * 获取url的hash和search值
   * @param url
   */
  window.getUrlHashSearchStr = function (url) {
    var hashStr = window.location.hash;
    var searchStr = window.location.search;
    if (url) {
      var urlHashIndex = url.indexOf('#');
      var urlSearchIndex = url.indexOf('?');
      hashStr = urlHashIndex !== -1 ? url.substr(urlHashIndex) : '';
      if (urlSearchIndex < urlHashIndex || (urlSearchIndex !== -1 && urlHashIndex === -1)) {
        var length = urlSearchIndex < urlHashIndex ? urlHashIndex - urlSearchIndex : url.length;
        searchStr = url.substr(urlSearchIndex, length);
      } else {
        searchStr = '';
      }
    }
    return {
      hashStr: hashStr,
      searchStr: searchStr
    };
  };

  /**
   * 将JSON对象拼接成GET URL形式参数
   * @param params
   * @returns {string}
   */
  window.getStitchingUrlParams = function (params) {
    var strParams = '';
    for (var keyItem in params) {
      var value = params[keyItem];
      if (value !== '') {
        strParams += keyItem + '=' + encodeURIComponent(typeof value === 'object' ? JSON.stringify(value) : value) + '&';
      }
    }
    return strParams ? strParams.substr(0, strParams.length - 1) : '';
  };

  /**
   * 格式化非法的正常的url 如 http://xxx.xx.com/?code=abcdefs#/
   * @returns {string}
   */
  window.getHttpUrlFormat = function (url) {
    var urlParams = window.getRequestUrlParam(url);
    var urlHashSearchStr = window.getUrlHashSearchStr(url);
    var hashStr = urlHashSearchStr.hashStr;
    var hashStrParams = hashStr.indexOf('?');
    url = url || window.location.href;
    // url是否有特殊类型的设定,如 http://xxx.xx.com/?code=abcdefs#/order?orderNo=123 需要重定向设置一下
    // 建议在跳转的时候，请采用  window.location.replace 的形式，这样可以删除当前无用的历史记录
    // 有 #/ 并且在之前有 ？号的情况下 就要重新格式化
    if (hashStr && urlHashSearchStr.searchStr) {
      var urlParamsStr = window.getStitchingUrlParams(urlParams);
      urlParamsStr = urlParamsStr ? (urlParamsStr.indexOf('?') !== -1 ? '&' : '?') + urlParamsStr : '';
      return window.location.origin + '/' + hashStr.substr(0, hashStrParams !== -1 ? hashStrParams : hashStr.length) + urlParamsStr;
    }
    return url;
  };

  /**
   * 获取 hash 类型 url path  路径
   * @param url {string} [url=window.location.href]
   * @returns {*}
   */
  window.getRequestHashPath = function (url) {
    var hashSearch = window.getUrlHashSearchStr(window.getHttpUrlFormat(url || window.location.href));
    var hashStr = hashSearch.hashStr;
    if (hashStr) {
      var paramsIndex = hashStr.indexOf('?');
      return hashStr.substr(0, paramsIndex !== -1 ? paramsIndex : hashStr.length);
    }
    return '';
  };

  /**
   * 获取纯粹的url地址,不要任何参数
   * @param url
   */
  window.getHttpSheerUrl = function (url) {
    var formatUrl = window.getHttpUrlFormat(url);
    if (formatUrl.indexOf('?') !== -1) {
      formatUrl = formatUrl.substr(0, formatUrl.indexOf('?'));
    }
    return formatUrl;
  };

}(window));
